
#include <iostream>
#include <cstring>

#define TYPELIST_1(T1)             Typelist<T1, NullType>
#define TYPELIST_2(T1, T2)         Typelist<T1, TYPELIST_1(T2) >
#define TYPELIST_3(T1, T2, T3)     Typelist<T1, TYPELIST_2(T2, T3) >
#define TYPELIST_4(T1, T2, T3, T4) Typelist<T1, TYPELIST_3(T2, T3, T4) >

struct NullType { };

template <class H, class T>
struct Typelist
{ 
    typedef H Head;
    typedef T Tail;
};

template <class TList, class T> struct IndexOf;

template <class T>
struct IndexOf< NullType, T>
{
    enum { value = -1 };
};

template <class T, class Tail>
struct IndexOf< Typelist<T, Tail>, T>
{
  enum { value = 0 };
};

template <class Head, class Tail, class T>
struct IndexOf< Typelist<Head, Tail>, T>
{
private:
    enum { temp = IndexOf<Tail, T>::value };
public:
    enum { value = (temp == -1) ? -1 : 1+temp };
};

typedef TYPELIST_4(char, int, long, double)  Pod_types;

template <typename T>
struct is_pod
{
    enum { value = IndexOf<Pod_types,T>::value != -1 };
};

template <typename T, bool IsPod>
struct copy_trait
{
  static  void copy( T* to, const T* from, int n)
  {
    std::cerr << "default copy with loop" << std::endl;
          
    for( int i = 0; i < n; ++i )
      to[i] = from[i];
  }
};
template <typename T>
struct copy_trait<T,true>
{
  static void copy( T* to, const T* from, int n)
  {
    std::cerr << "bitwise copy" << std::endl;
   
    memcpy( to, from, n*sizeof(T));
  }
};

template <typename T, typename Cpy = copy_trait<T, is_pod<T>::value> >
class matrix
{
public:
  matrix(int i, int j) : cols_(i), rows_(j), v_(new T[cols_*rows_]) {}
  ~matrix() {delete [] v_;}
  matrix(const matrix& rhs);
  matrix& operator=(const matrix& rhs);
private:
  int cols_;
  int rows_;
  T   *v_;

  void copy(const matrix&);
};
template <class T,typename Cpy>
matrix<T,Cpy>& matrix<T,Cpy>::operator=( const matrix &other)
{
  if ( this != &other )
  {
    delete [] v_;
    copy( other);
  }
  return *this;
}
template <class T, class Cpy>
void matrix<T,Cpy>::copy( const matrix &other)
{
  rows_ = other.rows_;
  cols_ = other.cols_;
  v_ = new T[rows_*cols_];
  Cpy::copy( v_, other.v_, rows_*cols_);
}
class nonpod
{
  char *ptr;
};

int main()
{
  matrix<int>    mi(10,10), mi2(10,10);
  matrix<double> md(20,10), md2(20,10);
  matrix<nonpod> mn(20,10), mn2(20,10);
  mi = mi2;
  md = md2;
  mn = mn2;

  return 0;
}
